@@ -1,34 +1,11 @@
-/*
- * Dropbear - a SSH2 server
- * 
- * Copyright (c) 2002,2003 Matt Johnston
- * All rights reserved.
- * 
- * Permission is hereby granted, free of charge, to any person obtaining a copy
- * of this software and associated documentation files (the "Software"), to deal
- * in the Software without restriction, including without limitation the rights
- * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
- * copies of the Software, and to permit persons to whom the Software is
- * furnished to do so, subject to the following conditions:
- * 
- * The above copyright notice and this permission notice shall be included in
- * all copies or substantial portions of the Software.
- * 
- * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
- * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
- * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
- * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
- * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
- * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
- * SOFTWARE. */
+#include "options.h"
+#include "libtomcrypt/mycrypt.h"
+#include "util.h"
+#include "buffer.h"
 
 #include "algo.h"
-#include "dbutil.h"
 
-/* This file (algo.c) organises the ciphers which can be used, and is used to
- * decide which ciphers/hashes/compression/signing to use during key exchange*/
-
-/* Mappings for ciphers, parameters are
+/* Mapping of ssh ciphers to libtomcrypt ciphers, with blocksizes etc.
    {&cipher_desc, keysize, blocksize} */
 
 #ifdef DROPBEAR_AES128_CBC
@@ -48,9 +25,8 @@ const struct dropbear_cipher dropbear_3des =
 	{&des3_desc, 24, 8};
 #endif
 
-/* used to indicate no encryption, as defined in rfc2410 */
 const struct dropbear_cipher dropbear_nocipher =
-	{NULL, 16, 8}; 
+	{NULL, 16, 8}; /* used initially */
 
 /* Mapping of ssh hashes to libtomcrypt hashes, including keysize etc.
    {&hash_desc, keysize, hashsize} */
@@ -83,7 +59,7 @@ algo_type sshciphers[] = {
 #ifdef DROPBEAR_3DES_CBC
 	{"3des-cbc", 0, (void*)&dropbear_3des, 1},
 #endif
-	{NULL, 0, NULL, 0}
+	{0}
 };
 
 algo_type sshhashes[] = {
@@ -93,7 +69,7 @@ algo_type sshhashes[] = {
 #ifdef DROPBEAR_MD5_HMAC
 	{"hmac-md5", 0, (void*)&dropbear_md5, 1},
 #endif
-	{NULL, 0, NULL, 0}
+	{0}
 };
 
 algo_type sshcompress[] = {
@@ -101,7 +77,7 @@ algo_type sshcompress[] = {
 #ifndef DISABLE_ZLIB
 	{"zlib", DROPBEAR_COMP_ZLIB, NULL, 1},
 #endif
-	{NULL, 0, NULL, 0}
+	{0}
 };
 
 algo_type sshhostkey[] = {
@@ -111,12 +87,12 @@ algo_type sshhostkey[] = {
 #ifdef DROPBEAR_DSS
 	{"ssh-dss", DROPBEAR_SIGNKEY_DSS, NULL, 1},
 #endif
-	{NULL, 0, NULL, 0}
+	{0}
 };
 
 algo_type sshkex[] = {
 	{"diffie-hellman-group1-sha1", DROPBEAR_KEX_DH_GROUP1, NULL, 1},
-	{NULL, 0, NULL, 0}
+	{0}
 };
 
 
@@ -137,12 +113,13 @@ void crypto_init() {
 #ifdef DROPBEAR_3DES_CBC
 		&des3_desc,
 #endif
-		NULL
+	NULL
 	};
 
 	const struct _hash_descriptor *reghashes[] = {
-		/* we need sha1 for hostkey stuff regardless */
+#ifdef DROPBEAR_SHA1_HMAC
 		&sha1_desc,
+#endif
 #ifdef DROPBEAR_MD5_HMAC
 		&md5_desc,
 #endif
@@ -161,64 +138,58 @@ void crypto_init() {
 			dropbear_exit("error registering crypto");
 		}
 	}
+
+	if (register_prng(&yarrow_desc) == -1) {
+		dropbear_exit("error registering crypto");
+	}
 }
 
-/* algolen specifies the length of algo, algos is our local list to match
- * against.
- * Returns DROPBEAR_SUCCESS if we have a match for algo, DROPBEAR_FAILURE
- * otherwise */
-int have_algo(char* algo, size_t algolen, algo_type algos[]) {
+/* returns 0 if we have a match for algo, -1 otherwise */
+int have_algo(char* algo, int algolen, algo_type algos[]) {
 
-	int i;
-
-	for (i = 0; algos[i].name != NULL; i++) {
+	int i = 0;
+	while (algos[i].name != NULL) {
 		if (strlen(algos[i].name) == algolen
 				&& (strncmp(algos[i].name, algo, algolen) == 0)) {
-			return DROPBEAR_SUCCESS;
+			return 0;
 		}
+		i++;
 	}
 
-	return DROPBEAR_FAILURE;
+	return -1;
 }
 
 
-/* match the first algorithm in the comma-separated list in buf which is
- * also in localalgos[], or return NULL on failure.
- * (*goodguess) is set to 1 if the preferred client/server algos match,
- * 0 otherwise. This is used for checking if the kexalgo/hostkeyalgos are
- * guessed correctly */
-algo_type * buf_match_algo(buffer* buf, algo_type localalgos[], int *goodguess)
-{
+
+/* hashlist is a comma seperated, null terminated list of hashes.
+ * The first item in the list which also has a local entry will be chosen */
+algo_type * buf_match_algo(buffer* buf, algo_type localalgos[]) {
 
 	unsigned char * algolist = NULL;
+	unsigned char* alterlist = NULL;
 	unsigned char * remotealgos[MAX_PROPOSED_ALGO];
+	unsigned int pos1, pos2;
 	unsigned int len;
 	unsigned int count, i, j;
-	algo_type * ret = NULL;
+	algo_type * ret;
 
-	/* get the comma-separated list from the buffer ie "algo1,algo2,algo3" */
+	/* get the comma-seperated list from the buffer ie "algo1,algo2,algo3" */
 	algolist = buf_getstring(buf, &len);
-	/* Debug this */
-	TRACE(("buf_match_algo: %s", algolist));
 	if (len > MAX_PROPOSED_ALGO*(MAX_NAME_LEN+1)) {
+		ret = NULL;
 		goto out; /* just a sanity check, no other use */
 	}
+	alterlist = strdup((char*)algolist);
 
 	/* remotealgos will contain a list of the strings parsed out */
-	/* We will have at least one string (even if it's just "") */
-	remotealgos[0] = algolist;
-	count = 1;
-	/* Iterate through, replacing ','s with NULs, to split it into
-	 * words. */
-	for (i = 0; i < len; i++) {
-		if (algolist[i] == '\0') {
-			/* someone is trying something strange */
-			goto out;
-		}
-		if (algolist[i] == ',') {
-			algolist[i] = '\0';
-			remotealgos[count] = &algolist[i+1];
+	count = 0;
+	pos1 = 0;
+	for (pos2 = 0; pos2 <= len; pos2++) {
+		if (alterlist[pos2] == ',' || alterlist[pos2] == '\0') {
+			remotealgos[count] = &alterlist[pos1];
+			alterlist[pos2] = '\0';
 			count++;
+			pos1 = pos2+1; /* this is safe since we check pos2 next loop */
 		}
 		if (count == MAX_PROPOSED_ALGO) {
 			break;
@@ -227,33 +198,27 @@ algo_type * buf_match_algo(buffer* buf, algo_type localalgos[], int *goodguess)
 
 	/* iterate and find the first match */
 	for (i = 0; i < count; i++) {
-
 		len = strlen(remotealgos[i]);
-
 		for (j = 0; localalgos[j].name != NULL; j++) {
-			if (localalgos[j].usable) {
-				if (len == strlen(localalgos[j].name) &&
-						strncmp(localalgos[j].name, remotealgos[i], len) == 0) {
-					/* set if it was a good guess */
-					if (i == 0 && j == 0) {
-						*goodguess = 1;
-					} else {
-						*goodguess = 0;
-					}
-					/* set the algo to return */
-					ret = &localalgos[j];
-					goto out;
-				}
+			if (!localalgos[j].usable) {
+				continue;
+			}
+			if (len == strlen(localalgos[j].name) 
+					&& strcmp(localalgos[j].name, remotealgos[i]) == 0) {
+				ret = &localalgos[j];
+				goto out;
 			}
 		}
 	}
+	ret = NULL;
 
 out:
+	m_free(alterlist);
 	m_free(algolist);
 	return ret;
 }
 
-/* Output a comma separated list of algorithms to a buffer */
+/* output a comma seperated list of algorithms to a buffer */
 void buf_put_algolist(buffer * buf, algo_type localalgos[]) {
 
 	unsigned int pos = 0, i, len;
@@ -261,16 +226,12 @@ void buf_put_algolist(buffer * buf, algo_type localalgos[]) {
 
 	for (i = 0; localalgos[i].name != NULL; i++) {
 		if (localalgos[i].usable) {
-			/* Avoid generating a trailing comma */
-			if (pos)
-			    str[pos++] = ',';
 			len = strlen(localalgos[i].name);
 			memcpy(&str[pos], localalgos[i].name, len);
 			pos += len;
+			str[pos] = ',';
+			pos++;
 		}
 	}
-	str[pos]=0;
-	/* Debug this */
-	TRACE(("buf_put_algolist: %s", str));
-	buf_putstring(buf, str, pos);
+	buf_putstring(buf, str, pos-1);
 }
