@@ -1,52 +1,24 @@
-/*
- * Dropbear - a SSH2 server
- * 
- * Copyright (c) 2002,2003 Matt Johnston
- * All rights reserved.
- * 
- * Permission is hereby granted, free of charge, to any person obtaining a copy
- * of this software and associated documentation files (the "Software"), to deal
- * in the Software without restriction, including without limitation the rights
- * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
- * copies of the Software, and to permit persons to whom the Software is
- * furnished to do so, subject to the following conditions:
- * 
- * The above copyright notice and this permission notice shall be included in
- * all copies or substantial portions of the Software.
- * 
- * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
- * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
- * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
- * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
- * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
- * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
- * SOFTWARE. */
-
-#include "includes.h"
+#include "options.h"
 #include "packet.h"
 #include "buffer.h"
 #include "session.h"
-#include "dbutil.h"
+#include "util.h"
 #include "channel.h"
 #include "chansession.h"
 #include "sshpty.h"
 #include "termcodes.h"
 #include "ssh.h"
 #include "random.h"
-#include "utmp.h"
-#include "x11fwd.h"
-#include "agentfwd.h"
-
-/* Handles sessions (either shells or programs) requested by the client */
 
 static int sessioncommand(struct Channel *channel, struct ChanSess *chansess,
-		int iscmd, int issubsys);
+		char iscmd);
 static int sessionpty(struct ChanSess * chansess);
 static int sessionsignal(struct ChanSess *chansess);
 static int noptycommand(struct Channel *channel, struct ChanSess *chansess);
 static int ptycommand(struct Channel *channel, struct ChanSess *chansess);
 static int sessionwinchange(struct ChanSess *chansess);
 static void execchild(struct ChanSess *chansess);
+static void addnewvar(const char* param, const char* var);
 static void addchildpid(struct ChanSess *chansess, pid_t pid);
 static void sesssigchild_handler(int val);
 
@@ -55,7 +27,6 @@ struct SigMap {
 	char* name;
 };
 
-/* Mapping of signal values to ssh signal strings */
 struct SigMap signames[] = {
 	{SIGABRT, "ABRT"},
 	{SIGALRM, "ALRM"},
@@ -70,14 +41,12 @@ struct SigMap signames[] = {
 	{SIGTERM, "TERM"},
 	{SIGUSR1, "USR1"},
 	{SIGUSR2, "USR2"},
-	{0, NULL}
+	{0}
 };
 
 /* required to clear environment */
 extern char** environ;
 
-/* Set up the general chansession environment, in particular child-exit
- * handling */
 void chansessinitialise() {
 
 	struct sigaction sa_chld;
@@ -95,22 +64,23 @@ void chansessinitialise() {
 	
 }
 
-/* handler for childs exiting, store the state for return to the client */
 static void sesssigchild_handler(int val) {
 
 	int status;
 	pid_t pid;
-	unsigned int i;
+	int i;
 	struct ChanSess * chansess;
 	struct sigaction sa_chld;
-
+	
 	TRACE(("enter sigchld handler"));
 	while ((pid = waitpid(-1, &status, WNOHANG)) > 0) {
 
 		/* find the corresponding chansess */
 		for (i = 0; i < ses.childpidsize; i++) {
+			assert(pid > 1);
 			if (ses.childpids[i].pid == pid) {
 
+				assert(ses.childpids[i].chansess != NULL);
 				chansess = ses.childpids[i].chansess;
 				chansess->exited = 1;
 				if (WIFEXITED(status)) {
@@ -118,9 +88,7 @@ static void sesssigchild_handler(int val) {
 				}
 				if (WIFSIGNALED(status)) {
 					chansess->exitsignal = WTERMSIG(status);
-#ifndef AIX
 					chansess->exitcore = WCOREDUMP(status);
-#endif
 				} else {
 					/* we use this to determine how pid exited */
 					chansess->exitsignal = -1;
@@ -134,7 +102,6 @@ static void sesssigchild_handler(int val) {
 	TRACE(("leave sigchld handler"));
 }
 
-/* send the exitstatus to the client */
 void send_msg_chansess_exitstatus(struct Channel * channel,
 		struct ChanSess * chansess) {
 
@@ -153,7 +120,6 @@ void send_msg_chansess_exitstatus(struct Channel * channel,
 
 }
 
-/* send the signal causing the exit to the client */
 void send_msg_chansess_exitsignal(struct Channel * channel,
 		struct ChanSess * chansess) {
 
@@ -167,11 +133,13 @@ void send_msg_chansess_exitsignal(struct Channel * channel,
 
 	/* we check that we can match a signal name, otherwise
 	 * don't send anything */
-	for (i = 0; signames[i].name != NULL; i++) {
+	i = 0;
+	while (signames[i].name != 0) {
 		if (signames[i].signal == chansess->exitsignal) {
 			signame = signames[i].name;
 			break;
 		}
+		i++;
 	}
 
 	if (signame == NULL) {
@@ -190,7 +158,7 @@ void send_msg_chansess_exitsignal(struct Channel * channel,
 	encrypt_packet();
 }
 
-/* set up a session channel */
+/* set up a session type channel */
 void newchansess(struct Channel *channel) {
 
 	struct ChanSess *chansess;
@@ -205,6 +173,7 @@ void newchansess(struct Channel *channel) {
 	chansess->master = -1;
 	chansess->slave = -1;
 	chansess->tty = NULL;
+
 	chansess->term = NULL;
 	chansess->termw = 0;
 	chansess->termh = 0;
@@ -215,27 +184,13 @@ void newchansess(struct Channel *channel) {
 
 	channel->typedata = chansess;
 
-#ifndef DISABLE_X11FWD
-	chansess->x11fd = -1;
-	chansess->x11authprot = NULL;
-	chansess->x11authcookie = NULL;
-#endif
-
-#ifndef DISABLE_AGENTFWD
-	chansess->agentfd = -1;
-	chansess->agentfile = NULL;
-	chansess->agentdir = NULL;
-#endif
-
 }
 
-/* clean a session channel */
+/* clean up a session type channel */
 void closechansess(struct Channel *channel) {
 
 	struct ChanSess *chansess;
-	unsigned int i;
-	struct logininfo *li;
-
+	int i;
 	chansess = (struct ChanSess*)channel->typedata;
 
 	TRACE(("enter closechansess"));
@@ -246,32 +201,17 @@ void closechansess(struct Channel *channel) {
 
 	m_free(chansess->cmd);
 	m_free(chansess->term);
-
 	if (chansess->tty) {
-		/* write the utmp/wtmp login record */
-		li = login_alloc_entry(chansess->pid, ses.authstate.username,
-				ses.hostname, chansess->tty);
-		login_logout(li);
-		login_free_entry(li);
-
 		pty_release(chansess->tty);
-		m_free(chansess->tty);
 	}
-
-#ifndef DISABLE_X11FWD
-	x11cleanup(chansess);
-#endif
-
-#ifndef DISABLE_AGENTFWD
-	agentcleanup(chansess);
-#endif
+	m_free(chansess->tty);
 
 	/* clear child pid entries */
 	for (i = 0; i < ses.childpidsize; i++) {
 		if (ses.childpids[i].chansess == chansess) {
 			assert(ses.childpids[i].pid > 0);
-			TRACE(("closing pid %d", ses.childpids[i].pid));
-			TRACE(("exited = %d", chansess->exited));
+			TRACE(("closing pid %d\n", ses.childpids[i].pid));
+			TRACE(("exited = %d\n", chansess->exited));
 			ses.childpids[i].pid = -1;
 			ses.childpids[i].chansess = NULL;
 		}
@@ -282,8 +222,6 @@ void closechansess(struct Channel *channel) {
 	TRACE(("leave closechansess"));
 }
 
-/* Handle requests for a channel. These can be execution requests,
- * or x11/authagent forwarding. These are passed to appropriate handlers */
 void chansessionrequest(struct Channel *channel) {
 
 	unsigned char * type;
@@ -300,40 +238,30 @@ void chansessionrequest(struct Channel *channel) {
 	wantreply = buf_getbyte(ses.payload);
 
 	if (typelen > MAX_NAME_LEN) {
+		send_msg_channel_failure(channel);
+		m_free(type);
 		TRACE(("leave chansessionrequest: type too long")); /* XXX send error?*/
-		goto out;
+		return;
 	}
 
 	chansess = (struct ChanSess*)channel->typedata;
 	assert(chansess != NULL);
-	TRACE(("type is %s", type));
+	TRACE(("type is %s\n", type));
 
-	if (strcmp(type, "window-change") == 0) {
-		ret = sessionwinchange(chansess);
+	if (strcmp(type, "exec") == 0) {
+		ret = sessioncommand(channel, chansess, 1);
 	} else if (strcmp(type, "shell") == 0) {
-		ret = sessioncommand(channel, chansess, 0, 0);
+		ret = sessioncommand(channel, chansess, 0);
 	} else if (strcmp(type, "pty-req") == 0) {
 		ret = sessionpty(chansess);
-	} else if (strcmp(type, "exec") == 0) {
-		ret = sessioncommand(channel, chansess, 1, 0);
-	} else if (strcmp(type, "subsystem") == 0) {
-		ret = sessioncommand(channel, chansess, 1, 1);
-#ifndef DISABLE_X11FWD
-	} else if (strcmp(type, "x11-req") == 0) {
-		ret = x11req(chansess);
-#endif
-#ifndef DISABLE_AGENTFWD
-	} else if (strcmp(type, "auth-agent-req@openssh.com") == 0) {
-		ret = agentreq(chansess);
-#endif
+	} else if (strcmp(type, "window-change") == 0) {
+		ret = sessionwinchange(chansess);
 	} else if (strcmp(type, "signal") == 0) {
 		ret = sessionsignal(chansess);
 	} else {
-		/* etc, todo "env", "subsystem" */
+		/* etc, todo "env", "subsystem", "x11-req" */
 	}
 
-out:
-
 	if (wantreply) {
 		if (ret == 0) {
 			send_msg_channel_success(channel);
@@ -347,7 +275,7 @@ out:
 }
 
 
-/* Send a signal to a session's process as requested by the client*/
+/* returns 0 on success, 1 otherwise */
 static int sessionsignal(struct ChanSess *chansess) {
 
 	int sig = 0;
@@ -356,7 +284,7 @@ static int sessionsignal(struct ChanSess *chansess) {
 
 	if (chansess->pid == 0) {
 		/* haven't got a process pid yet */
-		return DROPBEAR_FAILURE;
+		return 1;
 	}
 
 	signame = buf_getstring(ses.payload, NULL);
@@ -374,23 +302,22 @@ static int sessionsignal(struct ChanSess *chansess) {
 
 	if (sig == 0) {
 		/* failed */
-		return DROPBEAR_FAILURE;
+		return 1;
 	}
 			
 	if (kill(chansess->pid, sig) < 0) {
-		return DROPBEAR_FAILURE;
+		return 1;
 	} 
 
-	return DROPBEAR_SUCCESS;
+	return 0;
 }
 
-/* Let the process know that the window size has changed, as notified from the
- * client. Returns DROPBEAR_SUCCESS or DROPBEAR_FAILURE */
+/* returns 0 on success, 1 on failure */
 static int sessionwinchange(struct ChanSess *chansess) {
 
 	if (chansess->master < 0) {
 		/* haven't got a pty yet */
-		return DROPBEAR_FAILURE;
+		return 1;
 	}
 			
 	chansess->termc = buf_getint(ses.payload);
@@ -401,12 +328,12 @@ static int sessionwinchange(struct ChanSess *chansess) {
 	pty_change_window_size(chansess->master, chansess->termr, chansess->termc,
 		chansess->termw, chansess->termh);
 
-	return DROPBEAR_FAILURE;
+	return 0;
 }
 
-/* Set up a session pty which will be used to execute the shell or program.
- * The pty is allocated now, and kept for when the shell/program executes.
- * Returns DROPBEAR_SUCCESS or DROPBEAR_FAILURE */
+
+
+/* returns 0 on success, 1 on failure */
 static int sessionpty(struct ChanSess * chansess) {
 
 	unsigned int termlen;
@@ -418,7 +345,7 @@ static int sessionpty(struct ChanSess * chansess) {
 	if (termlen > MAX_TERM_LEN) {
 		/* TODO send disconnect ? */
 		TRACE(("leave sessionpty: term len too long"));
-		return DROPBEAR_FAILURE;
+		return 1;
 	}
 	chansess->termc = buf_getint(ses.payload);
 	chansess->termr = buf_getint(ses.payload);
@@ -427,10 +354,12 @@ static int sessionpty(struct ChanSess * chansess) {
 
 	/* allocate the pty */
 	assert(chansess->master == -1); /* haven't already got one */
+	TRACE(("about to pty_allocate"));
 	if (pty_allocate(&chansess->master, &chansess->slave, namebuf, 64) == 0) {
 		TRACE(("leave sessionpty: failed to allocate pty"));
-		return DROPBEAR_FAILURE;
+		return 1;
 	}
+	TRACE(("pty_allocate success (probably)"));
 	
 	chansess->tty = (char*)strdup(namebuf);
 	if (!chansess->tty) {
@@ -438,8 +367,11 @@ static int sessionpty(struct ChanSess * chansess) {
 	}
 
 	pty_setowner(ses.authstate.pw, chansess->tty);
+	TRACE(("done setowner"));
 	pty_change_window_size(chansess->master, chansess->termr, chansess->termc,
 			chansess->termw, chansess->termh);
+	TRACE(("done windowsize"));
+	
 
 	/* Term modes */
 	/* We'll ignore errors and continue if we can't set modes.
@@ -448,26 +380,16 @@ static int sessionpty(struct ChanSess * chansess) {
 		unsigned char opcode;
 		unsigned int value;
 		const struct TermCode * termcode;
-		unsigned int len;
-
-		len = buf_getint(ses.payload);
-		if (len != ses.payload->len - ses.payload->pos) {
-			dropbear_exit("bad term mode string");
-		}
 
 		while (((opcode = buf_getbyte(ses.payload)) != 0x00) &&
 				opcode <= 159) {
-
-			/* must be before checking type, so that value is consumed even if
-			 * we don't use it */
-			value = buf_getint(ses.payload);
-
 			/* handle types of code */
 			if (opcode > MAX_TERMCODE) {
 				continue;
 			}
 			termcode = &termcodes[(unsigned int)opcode];
 			
+			value = buf_getint(ses.payload);
 
 			switch (termcode->type) {
 
@@ -513,20 +435,20 @@ static int sessionpty(struct ChanSess * chansess) {
 			}
 		}
 		if (tcsetattr(chansess->master, TCSANOW, &termio) < 0) {
-			dropbear_log(LOG_INFO, "error setting terminal attributes");
+			TRACE(("tcsetattr error"));
+		} else {
+			TRACE(("tcsetattr success"));
 		}
 	}
 
 	TRACE(("leave sessionpty"));
-	return DROPBEAR_SUCCESS;
+	return 0;
+	
 }
 
-/* Handle a command request from the client. This is used for both shell
- * and command-execution requests, and passes the command to
- * noptycommand or ptycommand as appropriate.
- * Returns DROPBEAR_SUCCESS or DROPBEAR_FAILURE */
+/* returns 0 on sucesss, 1 otherwise */
 static int sessioncommand(struct Channel *channel, struct ChanSess *chansess,
-		int iscmd, int issubsys) {
+		char iscmd) {
 
 	unsigned int cmdlen;
 
@@ -534,7 +456,7 @@ static int sessioncommand(struct Channel *channel, struct ChanSess *chansess,
 
 	if (chansess->cmd != NULL) {
 		/* TODO - send error - multiple commands? */
-		return DROPBEAR_FAILURE;
+		return 1;
 	}
 
 	if (iscmd) {
@@ -543,18 +465,7 @@ static int sessioncommand(struct Channel *channel, struct ChanSess *chansess,
 
 		if (cmdlen > MAX_CMD_LEN) {
 			/* TODO - send error - too long ? */
-			return DROPBEAR_FAILURE;
-		}
-		if (issubsys) {
-#ifdef SFTPSERVER_PATH
-			if ((cmdlen == 4) && strncmp(chansess->cmd, "sftp", 4) == 0) {
-				m_free(chansess->cmd);
-				chansess->cmd = strdup(SFTPSERVER_PATH);
-			} else 
-#endif
-			{
-				return DROPBEAR_FAILURE;
-			}
+			return 1;
 		}
 	}
 
@@ -567,9 +478,7 @@ static int sessioncommand(struct Channel *channel, struct ChanSess *chansess,
 	}
 }
 
-/* Execute a command and set up redirection of stdin/stdout/stderr without a
- * pty.
- * Returns DROPBEAR_SUCCESS or DROPBEAR_FAILURE */
+/* returns 0 on success, 1 on fail */
 static int noptycommand(struct Channel *channel, struct ChanSess *chansess) {
 
 	int infds[2];
@@ -581,15 +490,15 @@ static int noptycommand(struct Channel *channel, struct ChanSess *chansess) {
 
 	/* redirect stdin/stdout/stderr */
 	if (pipe(infds) != 0)
-		return DROPBEAR_FAILURE;
+		return 1;
 	if (pipe(outfds) != 0)
-		return DROPBEAR_FAILURE;
+		return 1;
 	if (pipe(errfds) != 0)
-		return DROPBEAR_FAILURE;
+		return 1;
 
 	pid = fork();
 	if (pid < 0)
-		return DROPBEAR_FAILURE;
+		return 1;
 
 	if (!pid) {
 		/* child */
@@ -597,17 +506,24 @@ static int noptycommand(struct Channel *channel, struct ChanSess *chansess) {
 		/* redirect stdin/stdout */
 #define FDIN 0
 #define FDOUT 1
-		if ((dup2(infds[FDIN], STDIN_FILENO) < 0) ||
-			(dup2(outfds[FDOUT], STDOUT_FILENO) < 0) ||
-			(dup2(errfds[FDOUT], STDERR_FILENO) < 0)) {
-			TRACE(("leave noptycommand: error redirecting FDs"));
-			return DROPBEAR_FAILURE;
+		if (dup2(infds[FDIN], STDIN_FILENO) < 0) {
+			TRACE(("leave sessioncommand: error redirecting stdin"));
+			return 1;
 		}
-
 		close(infds[FDOUT]);
 		close(infds[FDIN]);
+
+		if (dup2(outfds[FDOUT], STDOUT_FILENO) < 0) {
+			TRACE(("leave sessioncommand: error redirecting stdout"));
+			return 1;
+		}
 		close(outfds[FDIN]);
 		close(outfds[FDOUT]);
+
+		if (dup2(errfds[FDOUT], STDERR_FILENO) < 0) {
+			TRACE(("leave sessioncommand: error redirecting stderr"));
+			return 1;
+		}
 		close(errfds[FDIN]);
 		close(errfds[FDOUT]);
 
@@ -616,7 +532,7 @@ static int noptycommand(struct Channel *channel, struct ChanSess *chansess) {
 
 	} else {
 		/* parent */
-		TRACE(("continue noptycommand: parent"));
+		TRACE(("continue sessioncommand: parent"));
 		chansess->pid = pid;
 
 		/* add a child pid */
@@ -632,9 +548,13 @@ static int noptycommand(struct Channel *channel, struct ChanSess *chansess) {
 		ses.maxfd = MAX(ses.maxfd, channel->outfd);
 		ses.maxfd = MAX(ses.maxfd, channel->errfd);
 
-		if ((fcntl(channel->outfd, F_SETFL, O_NONBLOCK) < 0) ||
-			(fcntl(channel->infd, F_SETFL, O_NONBLOCK) < 0) ||
-			(fcntl(channel->errfd, F_SETFL, O_NONBLOCK) < 0)) {
+		if (fcntl(channel->outfd, F_SETFL, O_NONBLOCK) < 0) {
+			dropbear_exit("Couldn't set nonblocking");
+		}
+		if (fcntl(channel->infd, F_SETFL, O_NONBLOCK) < 0) {
+			dropbear_exit("Couldn't set nonblocking");
+		}
+		if (fcntl(channel->errfd, F_SETFL, O_NONBLOCK) < 0) {
 			dropbear_exit("Couldn't set nonblocking");
 		}
 	}
@@ -642,95 +562,55 @@ static int noptycommand(struct Channel *channel, struct ChanSess *chansess) {
 #undef FDOUT
 
 	TRACE(("leave noptycommand"));
-	return DROPBEAR_SUCCESS;
+	return 0;
 }
 
-/* Execute a command or shell within a pty environment, and set up
- * redirection as appropriate.
- * Returns DROPBEAR_SUCCESS or DROPBEAR_FAILURE */
+/* returns 0 on success, 1 on fail */
 static int ptycommand(struct Channel *channel, struct ChanSess *chansess) {
 
 	pid_t pid;
-	struct logininfo *li;
-#ifdef DO_MOTD
-	buffer * motdbuf = NULL;
-	int len;
-	struct stat sb;
-	char *hushpath = NULL;
-#endif
+	
 
 	TRACE(("enter ptycommand"));
 
-	/* we need to have a pty allocated */
-	if (chansess->master == -1 || chansess->tty == NULL) {
-		dropbear_log(LOG_WARNING, "no pty was allocated, couldn't execute");
-		return DROPBEAR_FAILURE;
-	}
-	
+	/* we already have a pty allocated */
+	assert(chansess->master != -1 && chansess->tty != NULL);
 	pid = fork();
 	if (pid < 0)
-		return DROPBEAR_FAILURE;
+		return 1;
 
-	if (pid == 0) {
+	if (!pid) {
 		/* child */
 		
 		/* redirect stdin/stdout/stderr */
 		close(chansess->master);
 
 		pty_make_controlling_tty(&chansess->slave, chansess->tty);
+		m_free(chansess->tty);
 		
-		if ((dup2(chansess->slave, STDIN_FILENO) < 0) ||
-			(dup2(chansess->slave, STDERR_FILENO) < 0) ||
-			(dup2(chansess->slave, STDOUT_FILENO) < 0)) {
-			TRACE(("leave ptycommand: error redirecting filedesc"));
-			return DROPBEAR_FAILURE;
+		if (dup2(chansess->slave, STDIN_FILENO) < 0) {
+			TRACE(("leave sessioncommand: error redirecting stdin"));
+			return 1;
+		}
+
+		if (dup2(chansess->slave, STDOUT_FILENO) < 0) {
+			TRACE(("leave sessioncommand: error redirecting stdout"));
+			return 1;
+		}
+
+		if (dup2(chansess->slave, STDERR_FILENO) < 0) {
+			TRACE(("leave sessioncommand: error redirecting stderr"));
+			return 1;
 		}
 
 		close(chansess->slave);
 
-		/* write the utmp/wtmp login record - must be after changing the
-		 * terminal used for stdout with the dup2 above */
-		li= login_alloc_entry(getpid(), ses.authstate.username,
-				ses.hostname, chansess->tty);
-		login_login(li);
-		login_free_entry(li);
-
-		m_free(chansess->tty);
-
-#ifdef DO_MOTD
-		if (ses.opts->domotd) {
-			/* don't show the motd if ~/.hushlogin exists */
-
-			/* 11 == strlen("/hushlogin\0") */
-			len = strlen(ses.authstate.pw->pw_dir) + 11; 
-
-			hushpath = m_malloc(len);
-			snprintf(hushpath, len, "%s/hushlogin", ses.authstate.pw->pw_dir);
-
-			if (stat(hushpath, &sb) < 0) {
-				/* more than a screenful is stupid IMHO */
-				motdbuf = buf_new(80 * 25);
-				if (buf_readfile(motdbuf, MOTD_FILENAME) == DROPBEAR_SUCCESS) {
-					buf_setpos(motdbuf, 0);
-					while (motdbuf->pos != motdbuf->len) {
-						len = motdbuf->len - motdbuf->pos;
-						len = write(STDOUT_FILENO, 
-								buf_getptr(motdbuf, len), len);
-						buf_incrpos(motdbuf, len);
-					}
-				}
-				buf_free(motdbuf);
-			}
-			m_free(hushpath);
-		}
-#endif /* DO_MOTD */
-
 		execchild(chansess);
 		/* not reached */
 
 	} else {
 		/* parent */
-		TRACE(("continue ptycommand: parent"));
+		TRACE(("continue sessioncommand: parent"));
 		chansess->pid = pid;
 
 		/* add a child pid */
@@ -749,13 +629,12 @@ static int ptycommand(struct Channel *channel, struct ChanSess *chansess) {
 	}
 
 	TRACE(("leave ptycommand"));
-	return DROPBEAR_SUCCESS;
+	return 0;
 }
 
-/* Add the pid of a child to the list for exit-handling */
 static void addchildpid(struct ChanSess *chansess, pid_t pid) {
 
-	unsigned int i;
+	int i;
 	for (i = 0; i < ses.childpidsize; i++) {
 		if (ses.childpids[i].pid == -1) {
 			break;
@@ -766,7 +645,6 @@ static void addchildpid(struct ChanSess *chansess, pid_t pid) {
 	if (i == ses.childpidsize) {
 		ses.childpids = (struct ChildPid*)m_realloc(ses.childpids,
 				sizeof(struct ChildPid) * ses.childpidsize+1);
-		ses.childpidsize++;
 	}
 	
 	ses.childpids[i].pid = pid;
@@ -774,65 +652,59 @@ static void addchildpid(struct ChanSess *chansess, pid_t pid) {
 
 }
 
-/* Clean up, drop to user privileges, set up the environment and execute
- * the command/shell. This function does not return. */
+
+
+
 static void execchild(struct ChanSess *chansess) {
 
 	char *argv[4];
-	char * usershell;
-	char * baseshell;
-	unsigned int i;
+	int i, len;
+	int ret;
 
 	/* wipe the hostkey */
 	sign_key_free(ses.opts->hostkey);
-	ses.opts->hostkey = NULL;
 
-	/* overwrite the prng state */
-	seedrandom();
+	/* clear the state of the prng */
+	initrandom();
 
-	/* close file descriptors except stdin/stdout/stderr
-	 * Need to be sure FDs are closed here to avoid reading files as root */
-	for (i = 3; i < (unsigned int)ses.maxfd; i++) {
-		if (m_close(i) == DROPBEAR_FAILURE) {
-			dropbear_exit("Error closing file desc");
-		}
+	/* close file descriptors except stdin/stdout/stderr */
+	for (i = 3; i < ses.maxfd; i++) {
+		/* close() can fail, we need to be sure fds are closed */
+		do {
+			ret = close(i);
+			if (ret < 0 && (errno != EINTR) && (errno != EBADF)) {
+				dropbear_exit("error closing file desc");
+			}
+		} while ((ret < 0) && (errno != EBADF));
 	}
 
+
 	/* clear environment */
 	/* if we're debugging using valgrind etc, we need to keep the LD_PRELOAD
-	 * etc. This is hazardous, so should only be used for debugging. */
-#ifndef DEBUG_VALGRIND
+	 * etc. this is hazardous, so should only be used for debugging. */
+#ifndef DEBUG_KEEP_ENV
 #ifdef HAVE_CLEARENV
 	clearenv();
 #else /* don't HAVE_CLEARENV */
-	/* Yay for posix. */
-	if (environ) {
-		environ[0] = NULL;
-	}
+	environ = (char**)m_malloc(ENV_SIZE * sizeof(char*));
+	environ[0] = NULL;
 #endif /* HAVE_CLEARENV */
-#endif /* DEBUG_VALGRIND */
+#endif /* DEBUG_KEEP_ENV */
 
-	/* We can only change uid/gid as root ... */
-	if (getuid() == 0) {
-
-		if ((setgid(ses.authstate.pw->pw_gid) < 0) ||
-			(initgroups(ses.authstate.pw->pw_name, 
-						ses.authstate.pw->pw_gid) < 0) ||
-			(setuid(ses.authstate.pw->pw_uid) < 0)) {
-			dropbear_exit("error changing user");
-		}
-	} else {
-		/* ... but if the daemon is the same uid as the requested uid, we don't
-		 * need to */
-
-		/* XXX - there is a minor issue here, in that if there are multiple
-		 * usernames with the same uid, but differing groups, then the
-		 * differing groups won't be set (as with initgroups()). The solution
-		 * is for the sysadmin not to give out the UID twice */
-		if (getuid() != ses.authstate.pw->pw_uid) {
-			dropbear_exit("couldn't	change user as non-root");
-		}
+	/* change user */
+	if (setgid(ses.authstate.pw->pw_gid) < 0) {
+		dropbear_exit("error changing user");
 	}
+#ifndef HACKCRYPT
+	if (initgroups(ses.authstate.pw->pw_name,
+				ses.authstate.pw->pw_gid) < 0) {
+		dropbear_exit("error changing user");
+	}
+#endif
+	if (setuid(ses.authstate.pw->pw_uid) < 0) {
+		dropbear_exit("error changing user");
+	}
+
 
 	/* set env vars */
 	addnewvar("USER", ses.authstate.pw->pw_name);
@@ -848,39 +720,21 @@ static void execchild(struct ChanSess *chansess) {
 		dropbear_exit("error changing directory");
 	}
 
-#ifndef DISABLE_X11FWD
-	/* set up X11 forwarding if enabled */
-	x11setauth(chansess);
-#endif
-#ifndef DISABLE_AGENTFWD
-	/* set up agent env variable */
-	agentset(chansess);
-#endif
-
-	/* an empty shell should be interpreted as "/bin/sh" */
-	if (ses.authstate.pw->pw_shell[0] == '\0') {
-		usershell = "/bin/sh";
-	} else {
-		usershell = ses.authstate.pw->pw_shell;
+	/* set up execution */
+	/* shell commandname - the filename portion only */
+	len = strlen(ses.authstate.pw->pw_shell);
+	for (i = len-1; i > 0; i--) {
+		if (ses.authstate.pw->pw_shell[i] == '/') {
+			i++;
+			break;
+		}
 	}
-
-	baseshell = basename(usershell);
-
-	if (chansess->cmd != NULL) {
-		argv[0] = baseshell;
-	} else {
-		/* a login shell should be "-bash" for "/bin/bash" etc */
-		int len = strlen(baseshell) + 2; /* 2 for "-" */
-		argv[0] = (char*)m_malloc(len);
-		snprintf(argv[0], len, "-%s", baseshell);
-	}
-
+	argv[0] = &ses.authstate.pw->pw_shell[i];
 	if (chansess->cmd != NULL) {
 		argv[1] = "-c";
 		argv[2] = chansess->cmd;
 		argv[3] = NULL;
 	} else {
-		/* construct a shell of the form "-bash" etc */
 		argv[1] = NULL;
 	}
 
@@ -890,8 +744,7 @@ static void execchild(struct ChanSess *chansess) {
 	dropbear_exit("child failed");
 }
 	
-/* add a new environment variable, allocating space for the entry */
-void addnewvar(const char* param, const char* var) {
+static void addnewvar(const char* param, const char* var) {
 
 	char* newvar;
 	int plen, vlen;
