@@ -1,40 +1,23 @@
-/*
- * Dropbear - a SSH2 server
- * 
- * Copyright (c) 2002,2003 Matt Johnston
- * All rights reserved.
- * 
- * Permission is hereby granted, free of charge, to any person obtaining a copy
- * of this software and associated documentation files (the "Software"), to deal
- * in the Software without restriction, including without limitation the rights
- * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
- * copies of the Software, and to permit persons to whom the Software is
- * furnished to do so, subject to the following conditions:
- * 
- * The above copyright notice and this permission notice shall be included in
- * all copies or substantial portions of the Software.
- * 
- * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
- * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
- * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
- * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
- * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
- * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
- * SOFTWARE. */
+#include <stdio.h>
+#include <stdlib.h>
+#include <unistd.h>
+#include <fcntl.h>
+#include <sys/types.h>
+#include <sys/stat.h>
 
-#include "includes.h"
-#include "dbutil.h"
+#include "options.h"
+#include "util.h"
 #include "signkey.h"
 #include "bignum.h"
 #include "random.h"
 #include "buffer.h"
 #include "gendss.h"
 #include "dss.h"
+#include "libtomcrypt/mycrypt.h"
 
+#define PSIZE 128 /* 1024 bit*/
 #define QSIZE 20 /* 160 bit */
 
-/* This is just a test */
-
 #ifdef DROPBEAR_DSS
 
 static void getq(dss_key *key);
@@ -50,13 +33,17 @@ dss_key * gen_dss_priv_key(unsigned int size) {
 	key = (dss_key*)m_malloc(sizeof(dss_key));
 
 	key->p = (mp_int*)m_malloc(sizeof(mp_int));
+	m_mp_init(key->p);
 	key->q = (mp_int*)m_malloc(sizeof(mp_int));
+	m_mp_init(key->q);
 	key->g = (mp_int*)m_malloc(sizeof(mp_int));
+	m_mp_init(key->g);
 	key->y = (mp_int*)m_malloc(sizeof(mp_int));
+	m_mp_init(key->y);
 	key->x = (mp_int*)m_malloc(sizeof(mp_int));
-	m_mp_init_multi(key->p, key->q, key->g, key->y, key->x, NULL);
+	m_mp_init(key->x);
 	
-	seedrandom();
+	initrandom();
 	
 	getq(key);
 	getp(key, size);
@@ -70,35 +57,42 @@ dss_key * gen_dss_priv_key(unsigned int size) {
 
 static void getq(dss_key *key) {
 
+	int result;
 	char buf[QSIZE];
 
 	/* 160 bit prime */
-	genrandom(buf, QSIZE);
-	buf[0] |= 0x80; /* top bit high */
-	buf[QSIZE-1] |= 0x01; /* bottom bit high */
+	result = 0;
+	do {
+		genrandom(buf, QSIZE);
+		buf[0] |= 0x01; /* top bit high */
+		buf[QSIZE-1] |= 0x80; /* bottom bit high */
 
-	if (mp_read_unsigned_bin(key->q, buf, QSIZE) != MP_OKAY) {
-		fprintf(stderr, "dss key generation failed\n");
-		exit(1);
-	}
+		if (mp_read_unsigned_bin(key->q, buf, QSIZE) != MP_OKAY) {
+			fprintf(stderr, "dss key generation failed\n");
+			exit(1);
+		}
 
-	/* 18 rounds are required according to HAC */
-	if (mp_prime_next_prime(key->q, 18, 0) != MP_OKAY) {
-		fprintf(stderr, "dss key generation failed\n");
-		exit(1);
-	}
+		/* check for prime */
+		if (is_prime(key->q, &result) != CRYPT_OK) {
+			fprintf(stderr, "dss key generation failed\n");
+			exit(1);
+		}
+
+	} while (!result);
 }
 
 static void getp(dss_key *key, unsigned int size) {
 
-	DEF_MP_INT(tempX);
-	DEF_MP_INT(tempC);
-	DEF_MP_INT(tempP);
-	DEF_MP_INT(temp2q);
+	mp_int tempX, tempC, tempP, temp2q;
 	int result;
 	unsigned char *buf;
+	unsigned int count;
 
-	m_mp_init_multi(&tempX, &tempC, &tempP, &temp2q, NULL);
+
+	m_mp_init(&tempX);
+	m_mp_init(&tempC);
+	m_mp_init(&tempP);
+	m_mp_init(&temp2q);
 
 
 	/* 2*q */
@@ -110,10 +104,12 @@ static void getp(dss_key *key, unsigned int size) {
 	buf = (unsigned char*)m_malloc(size);
 
 	result = 0;
+	count = 0;
 	do {
 		
 		genrandom(buf, size);
-		buf[0] |= 0x80; /* set the top bit high */
+		buf[0] |= 0x01; /* set the bottom bit low */
+		buf[size-1] |= 0x80; /*set top bit high */
 
 		/* X is a random mp_int */
 		if (mp_read_unsigned_bin(&tempX, buf, size) != MP_OKAY) {
@@ -138,38 +134,45 @@ static void getp(dss_key *key, unsigned int size) {
 			exit(1);
 		}
 
-		/* now check for prime, 5 rounds is enough according to HAC */
+		/* now check for prime */
 		/* result == 1  =>  p is prime */
-		if (mp_prime_is_prime(key->p, 5, &result) != MP_OKAY) {
+		if (is_prime(key->p, &result) != CRYPT_OK) {
 			fprintf(stderr, "dss key generation failed\n");
 			exit(1);
 		}
+		count++;
 	} while (!result);
 
-	mp_clear_multi(&tempX, &tempC, &tempP, &temp2q, NULL);
+	mp_clear(&tempX);
+	mp_clear(&tempC);
+	mp_clear(&temp2q);
+	mp_clear(&tempP);
 	m_free(buf);
 }
 
 static void getg(dss_key * key) {
 
-	DEF_MP_INT(div);
-	DEF_MP_INT(h);
-	DEF_MP_INT(val);
+	char printbuf[1000];
+	mp_int div, h, val, dummy;
 
-	m_mp_init_multi(&div, &h, &val, NULL);
+	m_mp_init(&div);
+	m_mp_init(&h);
+	m_mp_init(&val);
+	m_mp_init(&dummy);
 
-	/* get div=(p-1)/q */
+	/* get (p-1)/q */
 	if (mp_sub_d(key->p, 1, &val) != MP_OKAY) {
 		fprintf(stderr, "dss key generation failed\n");
 		exit(1);
 	}
-	if (mp_div(&val, key->q, &div, NULL) != MP_OKAY) {
+	if (mp_div(&val, key->q, &div, &dummy) != MP_OKAY) {
 		fprintf(stderr, "dss key generation failed\n");
 		exit(1);
 	}
 
 	/* initialise h=1 */
 	mp_set(&h, 1);
+
 	do {
 		/* now keep going with g=h^div mod p, until g > 1 */
 		if (mp_exptmod(&h, &div, key->p, key->g) != MP_OKAY) {
@@ -177,19 +180,25 @@ static void getg(dss_key * key) {
 			exit(1);
 		}
 
-		if (mp_add_d(&h, 1, &h) != MP_OKAY) {
+		if (mp_add_d(&h, 1, &dummy) != MP_OKAY) {
 			fprintf(stderr, "dss key generation failed\n");
 			exit(1);
 		}
+		mp_exch(&h, &dummy);
 	
-	} while (mp_cmp_d(key->g, 1) != MP_GT);
+	} while (mp_cmp_d(key->g, 1) <= 0);
 
-	mp_clear_multi(&div, &h, &val, NULL);
+	mp_toradix(key->g, printbuf, 10);
+
+	mp_clear(&div);
+	mp_clear(&h);
+	mp_clear(&val);
+	mp_clear(&dummy);
 }
 
 static void getx(dss_key *key) {
 
-	DEF_MP_INT(val);
+	mp_int val;
 	char buf[QSIZE];
 	
 	m_mp_init(&val);
@@ -200,7 +209,7 @@ static void getx(dss_key *key) {
 		if (mp_read_unsigned_bin(&val, buf, QSIZE) != MP_OKAY) {
 			fprintf(stderr, "dss key generation failed\n");
 		}
-	} while ((mp_cmp_d(&val, 1) == MP_GT) && (mp_cmp(&val, key->q) == MP_LT));
+	} while ((mp_cmp_d(&val, 1) > 1) && (mp_cmp(&val, key->q) < 1));
 
 	mp_copy(&val, key->x);
 	mp_clear(&val);
