@@ -1,14 +1,3 @@
-/* LibTomCrypt, modular cryptographic library -- Tom St Denis
- *
- * LibTomCrypt is a library that provides various cryptographic
- * algorithms in a highly modular and flexible manner.
- *
- * The library is free for all purposes without any express
- * guarantee it works.
- *
- * Tom St Denis, tomstdenis@iahu.ca, http://libtomcrypt.org
- */
-
 /* these are smaller routines written by Clay Culver.  They do the same function as the rsa_encrypt/decrypt 
  * except that they are used to RSA encrypt/decrypt a single value and not a packet.
  */
@@ -16,14 +5,14 @@ int rsa_encrypt_key(const unsigned char *inkey, unsigned long inlen,
                     unsigned char *outkey, unsigned long *outlen,
                     prng_state *prng, int wprng, rsa_key *key)
 {
-   unsigned char rsa_in[RSA_STACK], rsa_out[RSA_STACK];
+   unsigned char rsa_in[4096], rsa_out[4096];
    unsigned long x, y, rsa_size;
    int err;
 
-   _ARGCHK(inkey  != NULL);
+   _ARGCHK(inkey != NULL);
    _ARGCHK(outkey != NULL);
    _ARGCHK(outlen != NULL);
-   _ARGCHK(key    != NULL);
+   _ARGCHK(key != NULL);
    
    /* only allow keys from 64 to 256 bits */
    if (inlen < 8 || inlen > 32) {
@@ -52,9 +41,6 @@ int rsa_encrypt_key(const unsigned char *inkey, unsigned long inlen,
       return CRYPT_BUFFER_OVERFLOW;
    }
 
-   /* store header */
-   packet_store_header(outkey, PACKET_SECT_RSA, PACKET_SUB_ENC_KEY);
-
    /* now lets make the header */
    y = PACKET_SIZE;
    
@@ -67,13 +53,15 @@ int rsa_encrypt_key(const unsigned char *inkey, unsigned long inlen,
        outkey[y] = rsa_out[x];
    }
 
-   *outlen = y;
+   /* store header */
+   packet_store_header(outkey, PACKET_SECT_RSA, PACKET_SUB_ENC_KEY);
+
 #ifdef CLEAN_STACK
    /* clean up */
    zeromem(rsa_in, sizeof(rsa_in));
    zeromem(rsa_out, sizeof(rsa_out));
 #endif
-
+   *outlen = y;
    return CRYPT_OK;
 }
 
@@ -81,14 +69,14 @@ int rsa_decrypt_key(const unsigned char *in, unsigned long inlen,
                           unsigned char *outkey, unsigned long *keylen, 
                           rsa_key *key)
 {
-   unsigned char sym_key[MAXBLOCKSIZE], rsa_out[RSA_STACK];
+   unsigned char sym_key[MAXBLOCKSIZE], rsa_in[4096], rsa_out[4096];
    unsigned long x, y, z, i, rsa_size;
    int err;
 
-   _ARGCHK(in     != NULL);
+   _ARGCHK(in != NULL);
    _ARGCHK(outkey != NULL);
    _ARGCHK(keylen != NULL);
-   _ARGCHK(key    != NULL);
+   _ARGCHK(key != NULL);
 
    /* right key type? */
    if (key->type != PK_PRIVATE && key->type != PK_PRIVATE_OPTIMIZED) {
@@ -116,12 +104,16 @@ int rsa_decrypt_key(const unsigned char *in, unsigned long inlen,
    }
    y += 4;
 
+   /* read it in */
+   for (x = 0; x < rsa_size; x++, y++) {
+       rsa_in[x] = in[y];
+   }
+
    /* decrypt it */
    x = (unsigned long)sizeof(rsa_out);
-   if ((err = rsa_exptmod(in+y, rsa_size, rsa_out, &x, PK_PRIVATE, key)) != CRYPT_OK) {
+   if ((err = rsa_exptmod(rsa_in, rsa_size, rsa_out, &x, PK_PRIVATE, key)) != CRYPT_OK) {
       return err;
    }
-   y += rsa_size;
 
    /* depad it */
    z = (unsigned long)sizeof(sym_key);
@@ -141,6 +133,7 @@ int rsa_decrypt_key(const unsigned char *in, unsigned long inlen,
 #ifdef CLEAN_STACK
    /* clean up */
    zeromem(sym_key, sizeof(sym_key));
+   zeromem(rsa_in, sizeof(rsa_in));
    zeromem(rsa_out, sizeof(rsa_out));
 #endif
    *keylen = z;
@@ -152,16 +145,16 @@ int rsa_sign_hash(const unsigned char *in,  unsigned long inlen,
                         rsa_key *key)
 {
    unsigned long rsa_size, x, y;
-   unsigned char rsa_in[RSA_STACK], rsa_out[RSA_STACK];
+   unsigned char rsa_in[4096], rsa_out[4096];
    int err;
 
-   _ARGCHK(in     != NULL);
-   _ARGCHK(out    != NULL);
+   _ARGCHK(in != NULL);
+   _ARGCHK(out != NULL);
    _ARGCHK(outlen != NULL);
-   _ARGCHK(key    != NULL);
+   _ARGCHK(key != NULL);
    
    /* reject nonsense sizes */
-   if (inlen > (512/3) || inlen < 16) {
+   if (inlen < 16) {
       return CRYPT_INVALID_ARG;
    }
 
@@ -215,13 +208,13 @@ int rsa_verify_hash(const unsigned char *sig, unsigned long siglen,
                     const unsigned char *md, int *stat, rsa_key *key)
 {
    unsigned long rsa_size, x, y, z;
-   unsigned char rsa_in[RSA_STACK], rsa_out[RSA_STACK];
+   unsigned char rsa_in[4096], rsa_out[4096];
    int err;
 
-   _ARGCHK(sig  != NULL);
-   _ARGCHK(md   != NULL);
+   _ARGCHK(sig != NULL);
+   _ARGCHK(md != NULL);
    _ARGCHK(stat != NULL);
-   _ARGCHK(key  != NULL);
+   _ARGCHK(key != NULL);
 
    /* always be incorrect by default */
    *stat = 0;
@@ -247,12 +240,16 @@ int rsa_verify_hash(const unsigned char *sig, unsigned long siglen,
    }
    y += 4;
 
+   /* load the signature */
+   for (x = 0; x < rsa_size; x++, y++) {
+       rsa_in[x] = sig[y];
+   }
+
    /* exptmod it */
-   x = (unsigned long)sizeof(rsa_out);
-   if ((err = rsa_exptmod(sig+y, rsa_size, rsa_out, &x, PK_PUBLIC, key)) != CRYPT_OK) {
+   x = (unsigned long)sizeof(rsa_in);
+   if ((err = rsa_exptmod(rsa_in, rsa_size, rsa_out, &x, PK_PUBLIC, key)) != CRYPT_OK) {
       return err;
    }
-   y += rsa_size;
 
    /* depad it */
    z = (unsigned long)sizeof(rsa_in);
